# -*- coding: utf-8 -*-

"""Provides shared mixins for ORM classes."""

__all__ = [
    'PolymorphicBaseMixin',
    'PolymorphicMixin',
    'TouchMixin',
]

import logging
logger = logging.getLogger(__name__)

from datetime import datetime

from sqlalchemy import Column
from sqlalchemy import Unicode
from sqlalchemy.ext.declarative import declared_attr

from pyramid_basemodel import save as save_to_db

class PolymorphicBaseMixin(object):
    """Provides a dynamically generated ``__mapper_args__`` property for
      [single table inherited][] ORM classes::
      
      [single table inherited]: http://bit.ly/TBDmMx
    """
    
    discriminator = Column(u'type', Unicode(16))
    
    @declared_attr
    def __mapper_args__(self):
        """Set the ``polymorphic_identity`` value to the lower case class name."""
        
        return {
            'polymorphic_on': self.discriminator,
            'polymorphic_identity': self.__class__.__name__.lower()
        }
    

class PolymorphicMixin(object):
    """Provides a dynamically generated ``__mapper_args__`` property for
      [single table inherited][] ORM classes::
      
      [single table inherited]: http://bit.ly/TBDmMx
    """
    
    @declared_attr
    def __mapper_args__(self):
        """Set the ``polymorphic_identity`` value to the lower case class name."""
        
        return {'polymorphic_identity': self.__class__.__name__.lower()}
    


class TouchMixin(object):
    """Provides ``touch`` and ``propagate_touch`` methods."""
    
    def propagate_touch(self):
        """Override to propagate touch events to relations.
          
          Note that this event *should not* be  called in response to an
          SQLAlchemy ORM attribute modified event, as you can't reliably
          update relations in an attribute event handler.
        """
    
    def touch(self, propagate=True, now=None, save=None):
        """Update self.modified."""
        
        # Compose.
        if now is None:
            now = datetime.utcnow
        if save is None:
            save = save_to_db
        
        # Update self's modified date.
        self.modified = now()
        save(self)
        
        # Call propagate touch.
        if propagate:
            self.propagate_touch()
    




# -*- coding: utf-8 -*-

"""Provides a base ORM mixin for models that need a name and a url slug."""

__all__ = [
    'BaseSlugNameMixin',
]

import logging
logger = logging.getLogger(__name__)

import slugify

from sqlalchemy import exc as sa_exc
from sqlalchemy import inspect as sa_inspect
from sqlalchemy.ext import declarative 
from sqlalchemy.schema import Column
from sqlalchemy.types import Unicode

from .util import ensure_unique
from .util import generate_random_digest

from . import Session

class BaseSlugNameMixin(object):
    """ORM mixin class that provides ``slug`` and ``name`` properties, with a
      ``set_slug`` method to set the slug value from the name and a default
      name aware factory classmethod.
    """
    
    _max_slug_length = 64
    _slug_is_unique = True

    @property
    def __name__(self):
        return self.slug
    
    
    @declarative.declared_attr
    def slug(cls):
        """A url friendly slug, e.g.: `foo-bar`."""
        
        l = cls._max_slug_length
        is_unique = cls._slug_is_unique
        return Column(Unicode(l), nullable=False, unique=is_unique)

    @declarative.declared_attr
    def name(cls):
        """A human readable name, e.g.: `Foo Bar`."""
        
        l = cls._max_slug_length
        return Column(Unicode(l), nullable=False)

    def set_slug(self, candidate=None, **kwargs):
        """Generate and set a unique ``self.slug`` from ``self.name``.
          
          Setup::
          
              >>> from mock import MagicMock as Mock
              >>> mock_inspect = Mock()
              >>> mock_unique = Mock()
              >>> return_none = lambda: None
              >>> return_true = lambda: True
              >>> from sqlalchemy.types import Integer
              >>> from pyramid_basemodel import Base
              >>> class Model(Base, BaseSlugNameMixin):
              ...     __tablename__ = 'models'
              ...     id =  Column(Integer, primary_key=True)
              >>> inst = Model()
              >>> inst.query = Mock()
              
          If there's a slug and no name, it's a noop::
          
              >>> inst.name = None
              >>> inst.slug = 'slug'
              >>> inst.set_slug(unique=mock_unique)
              >>> mock_unique.called
              False
              >>> inst.slug
              'slug'
              
          If there is a slug and a name and the slug is the candidate, then it's a noop::
          
              >>> inst.slug = u'abc'
              >>> inst.name = u'Abc'
              >>> inst.set_slug(candidate=u'abc', inspect=mock_inspect,
              ...         unique=mock_unique)
              >>> mock_unique.called
              False
              >>> mock_inspect.called
              True
              >>> inst.slug
              u'abc'
              
          If there's no name, uses a random digest::
          
              >>> mock_unique = lambda *args: args[-1]
              >>> inst.slug = None
              >>> inst.name = None
              >>> inst.set_slug(unique=mock_unique)
              >>> len(inst.slug)
              32
          
          Otherwise slugifies the name::
          
              >>> inst.name = u'My nice name'
              >>> inst.set_slug(unique=mock_unique)
              >>> inst.slug
              u'my-nice-name'
          
          Appending n until the slug is unique::
          
              >>> mock_unique = lambda *args: u'{0}-1'.format(args[-1])
              >>> inst.slug = None
              >>> inst.set_slug(unique=mock_unique)
              >>> inst.slug
              u'my-nice-name-1'

          Truncates the slug::

              >>> mock_unique = Mock()
              >>> inst.name = u'a' * 95
              >>> inst.set_slug(unique=mock_unique)
              >>> len(mock_unique.call_args[0][3])
              61
        """
        
        # Compose.
        gen_digest = kwargs.get('gen_digest', generate_random_digest)
        inspect = kwargs.get('inspect', sa_inspect)
        session = kwargs.get('session', Session)
        to_slug = kwargs.get('to_slug', slugify.slugify)
        unique = kwargs.get('unique', ensure_unique)

        # Generate a candidate slug.
        if candidate is None:
            if self.name:
                candidate = to_slug(self.name)
            if not candidate:
                candidate = gen_digest(num_bytes=16)
        
        # Make sure it's not longer than 64 chars.
        l = self._max_slug_length 
        l_minus_ellipsis = l - 3
        candidate = candidate[:l]
        unique_candidate = candidate[:l_minus_ellipsis]

        # If there's no name, only set the slug if its not already set.
        if self.slug and not self.name:
            return
        
        # If there is a name and the slug matches it, then don't try and
        # reset (i.e.: we only want to set a slug if the name has changed).
        if self.slug and self.name:
            if self.slug == candidate:
                # XXX as long as the instance is pending, as otherwise
                # we skip checking uniqueness on unsaved instances.
                try:
                    insp = inspect(self)
                except sa_exc.NoInspectionAvailable:
                    pass
                else:
                    if insp.persistent or insp.detached:
                        return

        # Iterate until the slug is unique.
        with session.no_autoflush:
            slug = unique(self, self.query, self.__class__.slug, unique_candidate)

        # Finally set the unique slug value.
        self.slug = slug



# -*- coding: utf-8 -*-

"""Marker interfaces for models and containers."""

__all__ = [
    'IDeclarativeBase',
    'IModel',
    'IModelContainer',
]

from zope.interface import Attribute
from zope.interface import Interface

class IDeclarativeBase(Interface):
    """Implemented by the declarative base and all classes that inherit from it."""

class IModel(IDeclarativeBase):
    """Provided by models."""

class IModelContainer(Interface):
    """Provided by model containers."""




# -*- coding: utf-8 -*-

"""Shared utility functions for interacting with the data model."""

import logging
logger = logging.getLogger(__name__)

import os
from binascii import hexlify

def generate_random_digest(num_bytes=28, urandom=None, to_hex=None):
    """Generates a random hash and returns the hex digest as a unicode string.
      
      Defaults to sha224::
      
          >>> import hashlib
          >>> h = hashlib.sha224()
          >>> digest = generate_random_digest()
          >>> len(h.hexdigest()) == len(digest)
          True
      
      Pass in ``num_bytes`` to specify a different length hash::
      
          >>> h = hashlib.sha512()
          >>> digest = generate_random_digest(num_bytes=64)
          >>> len(h.hexdigest()) == len(digest)
          True
      
      Returns unicode::
      
          >>> type(digest) == type(u'')
          True
      
    """
    
    # Compose.
    if urandom is None:
        urandom = os.urandom
    if to_hex is None:
        to_hex = hexlify
    
    # Get random bytes.
    r = urandom(num_bytes)
    
    # Return as a unicode string.
    return unicode(to_hex(r))

def ensure_unique(self, query, property_, value, max_iter=30, gen_digest=None):
    """Takes a ``candidate`` value for a unique ``property_`` and iterates,
      appending an incremented integer until unique.
    """
    
    # Compose.
    if gen_digest is None:
        gen_digest = generate_random_digest
    
    # Unpack
    candidate = value
    
    # Iterate until the slug is unique.
    n = 0
    n_str = ''
    while True:
        # Keep trying slug, slug-1, slug-2, etc.
        value = u'{0}{1}'.format(candidate, n_str)
        existing = None
        existing_instances = query.filter(property_==value).all()
        for instance in existing_instances:
            if instance != self:
                existing = instance
                break
        if existing and n < 30:
            n += 1
            # If we've tried 1, 2 ... all the way to ``max_iter``, then
            # fallback on appending a random digest rather than a sequential 
            # number.
            suffix = str(n) if n < 20 else gen_digest(num_bytes=8)
            n_str = u'-{0}'.format(suffix)
            continue
        break
    
    return value

def get_or_create(cls, **kwargs):
    """Get or create a ``cls`` instance using the ``kwargs`` provided.
      
          >>> from mock import Mock
          >>> mock_cls = Mock()
          >>> kwargs = dict(foo='bar')
      
      If an instance matches the filter kwargs, return it::
      
          >>> mock_cls.query.filter_by.return_value.first.return_value = 'exist'
          >>> get_or_create(mock_cls, **kwargs)
          'exist'
          >>> mock_cls.query.filter_by.assert_called_with(**kwargs)
      
      Otherwise return a new instance, initialised with the ``kwargs``::
      
          >>> mock_cls = Mock()
          >>> mock_cls.return_value = 'new'
          >>> mock_cls.query.filter_by.return_value.first.return_value = None
          >>> get_or_create(mock_cls, **kwargs)
          'new'
          >>> mock_cls.assert_called_with(**kwargs)
      
    """
    
    instance = cls.query.filter_by(**kwargs).first()
    if not instance:
        instance = cls(**kwargs)
    return instance

def get_all_matching(cls, column_name, values):
    """Get all the instances of ``cls`` where the column called ``column_name``
      matches one of the ``values`` provided.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_cls = Mock()
          >>> mock_cls.query.filter.return_value.all.return_value = ['result']
      
      Queries and returns the results::
      
          >>> get_all_matching(mock_cls, 'a', [1,2,3])
          ['result']
          >>> mock_cls.a.in_.assert_called_with([1,2,3])
          >>> mock_cls.query.filter.assert_called_with(mock_cls.a.in_.return_value)
      
    """
    
    column = getattr(cls, column_name)
    query = cls.query.filter(column.in_(values))
    return query.all()

def get_object_id(instance):
    """Return an identifier that's unique across database tables, e.g.::
      
          >>> from mock import MagicMock
          >>> mock_user = MagicMock()
          >>> mock_user.__tablename__ = 'users'
          >>> mock_user.id = 1234
          >>> get_object_id(mock_user)
          u'users#1234'
      
    """
    
    return u'{0}#{1}'.format(instance.__tablename__, instance.id)




# -*- coding: utf-8 -*-

"""Provides a base traversal root and a mixin class for objects in the
  Pyramid traversal hierarchy.
  
  http://docs.pylonsproject.org/projects/pyramid/en/latest/narr/traversal.html
"""

__all__ = [
    'BaseRoot',
]

import logging
logger = logging.getLogger(__name__)

from zope.interface import implementer
from zope.interface import alsoProvides

from pyramid.interfaces import ILocation

@implementer(ILocation)
class BaseRoot(object):
    """Base class for traversal factories."""
    
    __name__ = ''
    __parent__ = None
    
    def locatable(self, context, key, provides=None):
        """Make a context object locatable and return it."""
        
        # Compose.
        if provides is None:
            provides = alsoProvides
        
        if not hasattr(context, '__name__'):
            context.__name__ = key
        context._located_parent = self
        context.request = self.request
        if not ILocation.providedBy(context):
            provides(context, ILocation)
        return context
    
    def __init__(self, request, key='', parent=None):
        self.__name__ = key
        self.__parent__ = parent
        self.request = request
    




# -*- coding: utf-8 -*-

"""Provides a generic model class for storing large binary objects, e.g.:
  
  To store a bytestring::
  
      blob = Blob.factory('foo') # sets the blob's name to 'foo'
      blob.value = b'baz'
      save(blob)
  
  To store a file::
  
      f = open('/tmp/foo.pdf')
      blob = Blob.factory('foo', file_like_object=f)
      save(blob)
  
  To store a download from a url::
  
      blob = Blob.factory('foo')
      blob.update_from_url('http://www.example.com/foo.pdf')
      save(blob)
  
  The Blob's ``value`` is a read only buffer, e.g. to iterate over the contents::
  
      while True:
          chunk = blob.value.read(1024)
          if chunk is None:
              break
          # do something with ``chunk``
  
"""

__all__ = [
    'Blob',
]

import logging
logger = logging.getLogger(__name__)

try: # py2
    from cStringIO import StringIO
except ImportError: # py3
    from io import BytesIO

from gzip import GzipFile
from tempfile import NamedTemporaryFile

import requests as requests_lib

from sqlalchemy.schema import Column
from sqlalchemy.types import Unicode
from sqlalchemy.types import LargeBinary

from . import Base
from . import BaseMixin

class Blob(Base, BaseMixin):
    """Encapsulates a large binary file.
    
      Instances must have a unique ``self.name``, which has a maximum length
      of 64 characters.
      
      The binary data is stored in ``self.value``, either directly by assigning
      a bytestring, or passing a file like object to ``self.update()`` or by
      downloading a file from a url using ``self.update_from_url()``. The
      downloaded file can optionally be unzipped if compressed using gzip.
      
      When reading the data, ``self.value`` is a read only buffer. A convienience
      ``self.get_as_named_tempfile()`` method is provided as an easy way to get
      the value as a readable and writable file that can optionally be closed
      so the data is available from the filesystem.
    """
    
    __tablename__ = 'blobs'
    
    name = Column(Unicode(64), nullable=False, unique=True)
    value = Column(LargeBinary, nullable=False)
    
    @classmethod
    def factory(cls, name, file_like_object=None):
        """Create and return."""
        
        instance = cls()
        instance.update(name, file_like_object=file_like_object)
        return instance
    
    def update(self, name, file_like_object=None):
        """Update properties, reading the ``file_like_object`` into
          ``self.value`` if provided.
        """
        
        self.name = name
        if file_like_object is not None:
            self.value = file_like_object.read()
    
    def update_from_url(self, url, should_unzip=False, requests=None,
            gzip_cls=None, io_cls=None):
        """Update ``self.value`` to be the contents of the file downloaded
          from the ``url`` provided.
        """
        
        # Compose.
        if requests is None:
            requests = requests_lib
        if gzip_cls is None:
            gzip_cls = GzipFile
        if io_cls is None:
            io_cls = StringIO
        
        # Download the file, raising an exception if the download fails
        # after retrying once.
        attempts = 0
        max_attempts = 2
        while True:
            attempts += 1
            r = requests.get(url)
            if r.status_code == requests.codes.ok:
                break
            if attempts < max_attempts:
                continue
            r.raise_for_status()
        
        # If necessary, unzip using the gzip library.
        if should_unzip:
            self.value = gzip_cls(fileobj=io_cls(r.content)).read()
        else: # Read the response into ``self.value``.
            self.value = r.content
    
    def get_as_named_tempfile(self, should_close=False, named_tempfile_cls=None):
        """Read ``self.value`` into and return a named temporary file."""
        
        # Compose.
        if named_tempfile_cls is None:
            named_tempfile_cls = NamedTemporaryFile
        
        # Prepare the temp file.
        f = NamedTemporaryFile(delete=False)
        
        # Read self.value into it.
        if self.value is not None:
            f.write(self.value)
        
        # Close the file so its readable from the filename.
        if should_close:
            f.close()
        
        # Return the file.
        return f
    
    def __json__(self):
        return {
            'name': self.name
        }
    




# -*- coding: utf-8 -*-

"""Provides a base model container, used by the Pyramid traversal
  machinery and a mixin to aid with traversal from an instance
  back up the tree.
"""

__all__ = [
    'BaseModelContainer',
    'InstanceTraversalMixin',
]

import logging
logger = logging.getLogger(__name__)

import re
valid_slug = re.compile(r'^[.\w-]{1,64}$', re.U)

from zope.interface import implementer
from zope.interface import alsoProvides

from sqlalchemy.exc import InvalidRequestError

from pyramid.interfaces import ILocation
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Allow, Deny
from pyramid.security import Authenticated, Everyone

from . import Session
from .interfaces import IModelContainer
from .root import BaseRoot

def slug_validator(node, value, regexp=None):
    """Default to using a slug regexp."""
    
    # Compose.
    if regexp is None:
        regexp = valid_slug
    
    # Raise a ValueError.
    if not regexp.match(value):
        raise ValueError(u'{0} is not a valid slug.'.format(value))


@implementer(IModelContainer)
class BaseModelContainer(BaseRoot):
    """Traversal factory that looks up model classes by property."""
    
    property_name = 'slug'
    validation_exception = Exception
    
    @property
    def _validator(self):
        return slug_validator
    
    
    # Default container acl to be private whilst granting authenticated
    # users create permission.
    __acl__ = [
        (Allow, 'r:admin', ALL_PERMISSIONS),
        (Allow, Authenticated, 'view'),
        (Allow, Authenticated, 'create'),
        (Deny, Everyone, ALL_PERMISSIONS),
    ]
    
    @property
    def name(self):
        return self.model_cls.plural_class_name
    
    @property
    def class_name(self):
        return self.model_cls.class_name
    
    @property
    def plural_class_name(self):
        return self.model_cls.plural_class_name
    
    @property
    def class_slug(self):
        return self.model_cls.class_slug
    
    def get_child(self, key):
        """Query for and return the child instance, if found."""
        
        column = getattr(self.model_cls, self.property_name)
        query = self.model_cls.query.filter(column==key)
        return query.first()
    
    def __getitem__(self, key):
        """Lookup model instance by key."""
        
        print 'ERE getting', key
        try:
            print 'a'
            self.validator(None, key)
        except self.validation_exception as err:
            print 'b', err
            raise KeyError(key)
        
        print 'c'

        context = self.get_child(key)
        if not context:
            print 'd'
            raise KeyError(key)
        
        print 'e'
        return self.locatable(context, key)
    
    def __init__(self, request, model_cls, key=None, parent=None, **kwargs):
        """Instantiate the container."""
        
        # Compose.
        if key is None:
            key = model_cls.class_slug
        if parent is None:
            parent = BaseRoot(request)
        
        self.request = request
        self.model_cls = model_cls
        self.__name__ = key
        self.__parent__ = parent
        if kwargs.has_key('property_name'):
            self.property_name = kwargs.get('property_name')
        if kwargs.has_key('validator'):
            self.validator = kwargs.get('validator')
        else:
            self.validator = self._validator
    

class InstanceTraversalMixin(object):
    """Provide a default __parent__ implementation for traversal."""
    
    request = None
    traversal_key_name = 'slug'
    validation_exception = Exception
    
    @property
    def _validator(self):
        return slug_validator
    
    @property
    def _base_child_query(self):
        return self.query
    
    
    def get_container(self):
        """Reverse up the parent traversal hierarchy until reaching a container."""
        
        target = self
        while True:
            parent = target.__parent__
            if not parent:
                return None
            if IModelContainer.providedBy(parent):
                return parent
            target = parent
    
    def locatable(self, context, key, provides=None):
        """Make a context object locatable and pass on the request."""
        
        # Compose.
        if provides is None:
            provides = alsoProvides
        
        if not hasattr(context, '__name__'):
            context.__name__ = key
        context._located_parent = self
        context.request = self.request
        if not ILocation.providedBy(context):
            provides(context, ILocation)
        return context
    
    
    @property
    def __parent__(self, container_cls=None, session=None):
        """Either return ``self.parent``, or a model container object."""
        
        # Compose.
        if container_cls is None:
            container_cls = BaseModelContainer
        if session is None:
            session = Session
        
        # If the context has been located, return the container.
        if hasattr(self, '_located_parent'):
            return self._located_parent
        
        # Add self to the session to avoid ``DetachedInstanceError``s.
        session.add(self)
        
        # If the model has a parent, return it.
        parent = getattr(self, 'parent', None)
        if parent:
            return parent
        
        # Otherwise instantiate a "fake" traversal container and return that.
        # It's "fake" because it doesn't know about it's parent and doesn't
        # have a copy of the request.
        container = container_cls(None, self.__class__)
        return container
    
    def __getitem__(self, key):
        """Lookup model instance by key."""
        
        try:
            self._validator(None, key)
        except self.validation_exception:
            raise KeyError(key)
        
        # Only lookup children from instances that have them.
        has_children = hasattr(self, 'children')
        if not has_children:
            raise KeyError(key)
        
        # Only lookup if the target column exists.
        column = getattr(self.__class__, self.traversal_key_name, None)
        if not column:
            raise KeyError(key)
        
        try:
            query = self._base_child_query
            query = query.filter_by(parent=self).filter(column==key)
            context = query.first()
            if not context:
                raise KeyError(key)
        except InvalidRequestError as err:
            # If the query was invalid, the lookup fails, e.g.: if the
            # instance had the requisit properties but they weren't actually
            # sqlalchemy columns.
            logger.warn(err, exc_info=True)
            raise KeyError(key)
        
        # Return the context, having set the parent and flagged as locatable.
        return self.locatable(context, key)
    




# -*- coding: utf-8 -*-

"""Provide a ``BaseContentRoot`` traversal root for looking up instances
  via their containers.
"""

__all__ = [
    'BaseContentRoot',
]

import logging
logger = logging.getLogger(__name__)

from zope.interface import Interface
from zope.interface import alsoProvides

from .container import BaseModelContainer
from .root import BaseRoot

class BaseContentRoot(BaseRoot):
    """Base logic for looking up models."""
    
    apex = None # e.g.: (Design, IDesignsContainer, {})
    mapping = {} # {u'formats': (FileFormat, IFileFormatsContainer, {}), ...}
    
    def container_factory(self, item, key, provides=None, default_cls=None,
            interface_cls=None):
        """Return an instantiated and interface providing container."""
        
        # Compose.
        if provides is None:
            provides = alsoProvides
        if default_cls is None:
            default_cls = BaseModelContainer
        if interface_cls is None:
            interface_cls = Interface
        
        # Unpack the mapping item.
        model_cls, container_cls_or_interface, kwargs = item
        
        # If the container_cls_or_interface is an interface, then use the
        # default container cls and mark the instance as providing it.
        is_interface = issubclass(container_cls_or_interface, interface_cls)
        if is_interface:
            container_cls = default_cls
            container_interface = container_cls_or_interface
        else:
            container_cls = container_cls_or_interface
        
        # Instantiate the model container.
        container = container_cls(self.request, model_cls, key=key,
                parent=self, **kwargs)
        
        # Patch it to provide the specific container interface.
        if is_interface:
            provides(container, container_cls_or_interface)
        
        # Return the container.
        return container
    
    def __getitem__(self, key):
        """First see if the key is in ``self.mapping``. If it is, return
          a content container configured to look up that model class.
        """
        
        # If the key matches a traversal container in the mapping, use that.
        if key in self.mapping:
            mapping_item = self.mapping.get(key)
            return self.container_factory(mapping_item, key)
        
        # Otherwise try and lookup using the apex model class.
        if self.apex:
            container = self.container_factory(self.apex, '')
            return self.locatable(container[key], key)
        
        raise KeyError(key)
    




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

from os.path import dirname, join as join_path
from setuptools import setup, find_packages

def _read(file_name):
    sock = open(file_name)
    text = sock.read()
    sock.close()
    return text
    


requirements = [
    'pyramid_tm',
    'pyramid',
    'requests',
    'slugify',
    'zope.interface',
    'zope.sqlalchemy',
    'SQLAlchemy'
]
if sys.version_info[0] == 2:
    # currently has py3 compat issues
    requirements.append('inflect')

setup(
    name = 'pyramid_basemodel',
    version = '0.3.3',
    description = 'Global base classes for Pyramid SQLAlchemy applications.',
    author = 'James Arthur',
    author_email = 'username: thruflo, domain: gmail.com',
    url = 'http://github.com/thruflo/pyramid_basemodel',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Framework :: Pylons',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    license = _read('UNLICENSE').split('\n')[0],
    packages = find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe = False,
    install_requires=requirements
)
